# リスト2-7  データの処理をpandas側で行う場合

import pandas as pd
import numpy as np
# データ準備
# 2016年　一世帯当たりアイスクリーム支出金額（一般社団法人日本アイスクリーム協会）
# https://www.icecream.or.jp/data/expenditures.html
# 2016年　月別平均気温（気象庁）
# http://www.data.jma.go.jp/obd/stats/etrn/view/monthly_s3.php?
# prec_no=44&block_no=47662&view=a2
dfx = pd.DataFrame( [[1, 464, 10.6], [2, 397, 12.2], [3, 493, 14.9], \
    [4, 617, 20.3], [5, 890, 25.2], [6, 883, 26.3], [7, 1292, 29.7], \
    [8, 1387, 31.6], [9, 843, 27.7], [10, 621, 22.6], [11, 459, 15.5], \
    [12, 561, 13.8]], columns=['月', 'アイスクリーム支出', '平均気温'])
# 列の選択
dfcol = dfx['アイスクリーム支出']
print('アイスクリーム支出列の選択')
print(dfcol)
# 出力は
# 0         464
# 1         397
# (中略)
# 11        561
#
# 行の条件による選択
dfcond = dfx[dfx['平均気温'] >= 22]
print('平均気温>=22による行の選択')
print(dfcond)
# 出力は
#     月  アイスクリーム支出  平均気温
# 4   5        890  25.2
# 5   6        883  26.3
# 6   7       1292  29.7
# 7   8       1387  31.6
# 8   9        843  27.7
# 9  10        621  22.6
#
# 表のjoin結合 初めに月別アイスクリーム支出と月別平均気温の表を別々に作りjoinする
df1 = dfx[['月', 'アイスクリーム支出']]
df2 = dfx[['月', '平均気温']]
dfjoin = pd.merge(df1, df2, on='月', how='inner')  # 月をキーにしてdf1とdf2をjoinする
print('join結合')
print(dfjoin)
# 出力は
#      月  アイスクリーム支出  平均気温
# 0    1        464  10.6
# 1    2        397  12.2
#     ( 中略 )
# 11  12        561  13.8
#
# カウント
print('条件を満たす行のカウント')
print( (dfx['平均気温'] >= 22).sum())
# 出力は
# 6
#
# グルーピングして合計・平均
dfseason = pd.DataFrame(['春', '春', '春', '夏', '夏', '夏', '秋', '秋', '秋', \
    '冬', '冬', '冬'], columns=['季節'])
season_order = ['春', '夏', '秋', '冬']
dfx['季節'] = pd.Categorical(dfseason['季節'], season_order)  # Categoricalにするとprint時に個の順に並ぶ
dfg1 = dfx.groupby('季節')[['アイスクリーム支出']].sum().rename( \
    columns={'アイスクリーム支出': 'アイスクリーム支出合計'})
print('アイスクリーム支出の季節ごとの合計')
print(dfg1)
# 出力は
#     アイスクリーム支出合計
# 季節             
# 春          1354
# 夏          2390
# 秋          3522
# 冬          1641
#
dfg2 = dfx.groupby('季節')[['平均気温']].mean().rename(columns={'平均気温':'季節ごとの月別気温の平均'})
print('月別平均気温の季節ごとの平均')
print(dfg2)
# 出力は
#     季節ごとの月別気温の平均
# 季節              
# 春      12.566667
# 夏      23.933333
# 秋      29.666667
# 冬      17.300000
#
# グルーピングしてカウント
dfg3 = dfx.groupby('季節')[['月']].count()
print(dfg3)
# 出力は
#      月
# 季節   
# 春   3
# 夏   3
# 秋   3
# 冬   3
