# リスト3-17  5教科の成績の因子分析
#%matplotlib inline  # Jupyter Notebookで実行する場合コメントマーク#を外す
# -*- coding: utf-8 -*-
import math
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from sklearn.preprocessing import scale
from factor_analyzer import FactorAnalyzer
from sklearn.decomposition import FactorAnalysis

def biplot(score, coeff, pcax, pcay, labels=None):
# https://sukhbinder.wordpress.com/2015/08/05/biplot-with-python/よりアイデアを借用
    pca1 = pcax - 1
    pca2 = pcay - 1
    xs = score.iloc[:, pca1]
    ys = score.iloc[:, pca2]
    n=coeff.shape[0]
    scalex = 2.0 / (xs.max() - xs.min())
    scaley = 2.0 / (ys.max() - ys.min())
    for i in range(len(xs)):
        plt.text(xs[i] * scalex, ys[i] * scaley, str(i + 1), color='k', \
            ha='center', va='center')
    for i in range(n):
        plt.arrow(0, 0, coeff.iloc[i, pca1], coeff.iloc[i, pca2], color='r', \
            alpha=1.0)
        if labels is None:
            plt.text(coeff.iloc[i, pca1] * 1.10, coeff.iloc[i, pca2] * 1.10, \
                "Var" + str(i + 1), color='k', ha='center', va='center')
        else:
            plt.text(coeff.iloc[i, pca1] * 1.10, coeff.iloc[i, pca2] * 1.10, \
                labels[i], color='k', ha='center', va='center')
    plt.xlim(min(coeff.iloc[:, pca1].min() - 0.1, -1.1), \
        max(coeff.iloc[:, pca1].max() + 0.1, 1.1))
    plt.ylim(min(coeff.iloc[:, pca2].min() - 0.1, -1.1), \
        max(coeff.iloc[:, pca2].max() + 0.1, 1.1))
    plt.xlabel("F{}".format(pcax))
    plt.ylabel("F{}".format(pcay))
    plt.grid()
    plt.show()

subject = ['国語', '社会', '数学', '理科', '英語']
seiseki_a = np.array([
  [42, 49, 42, 35, 48], [35, 48, 45, 52, 46], [44, 52, 49, 38, 52],
  [42, 52, 43, 49, 46], [34, 47, 45, 46, 48], [43, 52, 46, 36, 48],
  [41, 39, 42, 39, 43], [62, 59, 59, 48, 54], [46, 44, 47, 39, 37],
  [77, 61, 48, 48, 67], [49, 55, 57, 48, 53], [48, 44, 42, 46, 60],
  [40, 38, 45, 49, 34], [36, 36, 44, 47, 47], [54, 50, 50, 45, 46],
  [52, 47, 61, 66, 46], [40, 52, 36, 47, 46], [63, 28, 35, 42, 48],
  [44, 33, 49, 20, 29], [46, 59, 50, 53, 57], [51, 41, 60, 59, 63],
  [45, 39, 48, 46, 45], [34, 39, 43, 50, 40], [34, 29, 45, 44, 48],
  [57, 46, 54, 46, 42], [38, 42, 41, 36, 41], [43, 47, 41, 53, 44],
  [45, 51, 53, 46, 53], [49, 56, 54, 61, 51], [35, 38, 57, 65, 57]
])
seiseki_in = pd.DataFrame(seiseki_a, columns=subject)
seiseki = pd.DataFrame(scale(seiseki_in), columns=seiseki_in.columns.values)

fa = FactorAnalyzer()
fa.analyze(seiseki, 2, rotation="varimax")  # varimax回転を用いるとき
# fa.analyze(seiseki, 2, rotation="promax")  # promax回転を用いるとき
# fa.analyze(seiseki, 2, rotation=None)  # 回転をしないとき

print('相関行列\n', seiseki.corr(method='pearson'))
print()
print('因子負荷量', fa.loadings.round(4))
print()
print('独自性', fa.get_uniqueness().round(4))
print()
print('因子分散', fa.get_factor_variance().round(4))
print()

##################
# 寄与率
kiyo = np.array([0, 0])
for i in range(len(fa.loadings)):
    u = np.array(fa.loadings.iloc[i])
    kiyo = kiyo + u * u
kiyo = pd.DataFrame(kiyo / len(fa.loadings), index=fa.loadings.columns.values).T
kiyo = kiyo.append(pd.DataFrame(np.cumsum(kiyo, axis=1)), \
    ignore_index=True).rename({0:'寄与率', 1:'累積寄与率'})
print('寄与率\n', kiyo)
print()

#################
def factor_score(X, load):
    Xs = pd.DataFrame(scale(X), columns=X.columns.values)
    ir = np.linalg.inv(Xs.corr(method='pearson'))
    return(pd.DataFrame(np.dot(Xs, np.dot(ir, load)), \
        columns=load.columns.values, index=X.index.values))

score = factor_score(seiseki, fa.loadings)
print('回帰法スコア\n', factor_score(seiseki, fa.loadings))
print()

biplot(score, fa.loadings, 1, 2, labels=subject)
