# リスト3-19  パッケージmcaを使ったコレスポンデンス分析の例
#%matplotlib inline  # Jupyter Notebookで実行する場合コメントマーク#を外す
# コレスポンデンス分析のライブラリ　髪の色・目の色の例
# hair/eye distribution https://www.utdallas.edu/~herve/Abdi-MCA2007-pretty.pdf
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from sklearn.preprocessing import OneHotEncoder
import mca
np.set_printoptions(formatter={'float':'{: 0.4f}'.format})
pd.set_option('display.precision', 5)
pd.set_option('display.max_columns', 25)

t = np.array([
  [326, 38, 241, 110, 3],
  [688, 116, 584, 188, 4],
  [343, 84, 909, 412, 26],
  [98, 48, 403, 681, 85]])
# データを1人ずつに展開して、OneHot形式に変換する
z = [[[i, j]] * (int(t[i, j])) for i in range(4) for j in range(5)]
tdata = []
for v in z:
    tdata.extend(v)
oh = OneHotEncoder()
oh_fit = oh.fit(tdata)
oh_ary = oh_fit.transform(tdata).toarray()
hair = ['Fair hair', 'Red hair','Medium hair', 'Dark hair', 'Black hair']
eyes = ['Blue eyes', 'Light eyes', 'Medium eyes', 'Dark eyes']
colindex = hair + eyes
X = pd.DataFrame(oh_ary, columns=colindex).astype(int)

# mcaでコレスポンデンス分析
ncols = 2  # hair, eye
mca_ben = mca.MCA(X, ncols=ncols)

# 固有値と貢献度
data = np.array([mca_ben.L[:2],
                 mca_ben.expl_var(greenacre=True, N=2) * 100]).T
df3 = pd.DataFrame(data=data, columns=['cλ','%c'], index=range(1, 3))
print('df3\n', df3)
# 各データ点の因子負荷量と貢献度
fs, cont ='Factor score', 'Contributions x 1000'
table3 = pd.DataFrame(columns=X.index, index=pd.MultiIndex\
                      .from_product([[fs, cont], range(1, 3)]))
table3.loc[fs, :] = mca_ben.fs_r(N=2).T
table3.loc[cont, :] = mca_ben.cont_r(N=2).T * 1000
print('table3\n', np.round(table3.astype(float), 2))

# 各データを主成分軸上に散布図でプロット
colors = {0: 'red', 1: 'orange', 2: 'green', 3: 'blue', 4: 'yellow'}
markers = {0: 'o', 1: '>', 2: '^', 3: 's'}
tdf = pd.DataFrame(tdata, columns=['eyes', 'hair'])
tdf['fs0'] = mca_ben.fs_r(N=2)[:, 0]
tdf['fs1'] = mca_ben.fs_r(N=2)[:, 1]
for u in [0, 1, 2, 3, 4]:
    for v in [0, 1, 2, 3]:
        if (u == 0) and (v == 0):
            ax = tdf[(tdf['hair'] == u) & (tdf['eyes'] == v)].plot.scatter( \
               x='fs0', y='fs1', s=50, c=colors[u], marker=markers[v])
        else:
            tdf[(tdf['hair'] == u) & (tdf['eyes'] == v)].plot.scatter( \
               x='fs0', y='fs1', s=50, c=colors[u], marker=markers[v], ax=ax)
plt.title('コレスポンデンス分析・髪と眼の色')
plt.margins(0.1)
plt.axhline(0, color='gray')
plt.axvline(0, color='gray')
plt.savefig('correspondence-haireyes-tutorialstyle.png')
plt.show()
