# リスト4-4  混合ガウス分布の生成と解析の例  人工発生データの解析
#%matplotlib inline  # Jupyter Notebookで実行する場合コメントマーク#を外す
import numpy as np
import matplotlib.pyplot as plt
from sklearn import mixture

# 正規分布(m=0, sd=1)で乱数を75000個生成
dist1 = np.random.normal(0, 1, 75000)
# 正規分布(m=3, sd=0.5)で乱数を25000個生成
dist2 = np.random.normal(3, 0.5, 25000)
dist = np.r_[dist1, dist2]  # 2つの配列をマージ
plt.hist(dist, bins=100)  # 100本のヒストグラムを作成
plt.title('正規分布 $N(0,1)$と$N(3,0.5)$を7500点対2500点で混合した分布')
plt.savefig('mixed-gaussian-plot-sample.png')
plt.show()  # グラフを表示

dist1 = np.reshape(dist, (np.shape(dist)[0], 1))  # distデータをnx1の配列に作る
max_n_components = 8
lowest_bic = np.infty
lowest_bic_ix = 0
for i in range(1, max_n_components):
    clf = mixture.GaussianMixture(n_components=i, covariance_type='full')
    clf.fit(dist1)
    bic = clf.bic(dist1)
    print('bic', i, bic)
    if bic < lowest_bic:
        lowest_bic = bic
        lowest_bic_ix = i
print('lowest bic case no', lowest_bic_ix)
clf = mixture.GaussianMixture(n_components=lowest_bic_ix, covariance_type='full')
clf.fit(dist1)
print('weights:', clf.weights_)
print('means:', clf.means_)
print('std dev:', np.sqrt(clf.covariances_))
