# リスト4-5  混合ガウス分布の生成と解析の例  irisデータ
#%matplotlib inline  # Jupyter Notebookで実行する場合コメントマーク#を外す
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.patches as patches
from sklearn.datasets import load_iris
from sklearn import mixture

iris = load_iris()
species = ['Setosa', 'Versicolour', 'Virginica']
irispddata = pd.DataFrame(iris.data, columns=iris.feature_names)
irispdtarget = pd.DataFrame(iris.target, columns=['target'])
irispd = pd.concat([irispddata, irispdtarget], axis=1)
max_n_components = 8
lowest_bic = np.infty
lowest_bic_ix = 0
for i in range(1, max_n_components):
    clf = mixture.GaussianMixture(n_components=i, covariance_type='full')
    clf.fit(irispddata)
    bic = clf.bic(irispddata)
    print('bic', i, bic)
    if bic < lowest_bic:
        lowest_bic = bic
        lowest_bic_ix = i
print('lowest bic case no', lowest_bic_ix)

clf = mixture.GaussianMixture(n_components=lowest_bic_ix, covariance_type='full')
# クラスター数を3にする場合は、
#　clf = mixture.GaussianMixture(n_components=3, covariance_type='full')
# とするとともに、プログラムの最後から5～10行目付近のコードのコメントを外して
# 図中に3番目の楕円を描く
clf.fit(irispddata)
print('weights:', clf.weights_)
print('means:', clf.means_)
print('std dev:', np.sqrt(clf.covariances_))

# 結果の表示
irispd0 = irispd[irispd['target'] == 0]
irispd1 = irispd[irispd['target'] == 1]
irispd2 = irispd[irispd['target'] == 2]
plt.scatter(irispd0['petal length (cm)'], irispd0['petal width (cm)'], c='red', \
    label=species[0], marker='x')
plt.scatter(irispd1['petal length (cm)'], irispd1['petal width (cm)'], c='blue', \
    label=species[1], marker='.')
plt.scatter(irispd2['petal length (cm)'], irispd2['petal width (cm)'], c='green', \
    label=species[2], marker='+')
ax = plt.axes()
c = patches.Ellipse(xy=(clf.means_[0, 2], clf.means_[0, 3]), \
                    width=np.sqrt(clf.covariances_[0, 2, 2]) * 4, \
                    height=np.sqrt(clf.covariances_[0, 3, 3]) * 4, \
                    fc='g', ec='r', alpha=0.2)
ax.add_patch(c)
d = patches.Ellipse(xy=(clf.means_[1, 2], clf.means_[1, 3]), \
                    width=np.sqrt(clf.covariances_[1, 2, 2]) * 4, \
                    height=np.sqrt(clf.covariances_[1, 3, 3]) * 4, \
                    fc='g', ec='r', alpha=0.2)
ax.add_patch(d)
# クラスター数を3にする場合は以下の6行のコメントマークを外し、第3の楕円を描く
# e = patches.Ellipse(xy=(clf.means_[2, 2], clf.means_[2, 3]), \
#                     width=np.sqrt(clf.covariances_[2, 2, 2]) * 4, \
#                     height=np.sqrt(clf.covariances_[2, 3, 3]) * 4, \
#                     fc='g', ec='r', alpha=0.2)
# ax.add_patch(e)

plt.title('irisデータの混合ガウス分布によるクラスター推定')
plt.xlabel('花弁の長さ (cm)')
plt.ylabel('花弁の幅 (cm)')
plt.show()

