# リスト4-9  フィッシャーのアヤメのデータ（花弁の長さ・幅）をSVMで分類するプログラム例
#%matplotlib inline  # Jupyter Notebookで実行する場合コメントマーク#を外す
# 出典は下記を改変。
#   http://scikit-learn.org/stable/auto_examples/svm/plot_iris.html#sphx-glr-auto-examples-svm-plot-iris-py
import numpy as np
import matplotlib.pyplot as plt
from sklearn import svm, datasets
iris = datasets.load_iris()
# フィッシャーのアヤメのデータのうち、花弁の長さと花弁の幅のみ使うことにする
X = iris.data[:, :2]
y = iris.target

h = .02  # メッシュの間隔
C = 1.0  # SVMを制御するパラメータ
svc = svm.SVC(kernel='linear', gamma='auto', C=C).fit(X, y)  # SVCクラスでlinearを選択
rbf_svc = svm.SVC(kernel='rbf', gamma=0.7, C=C).fit(X, y)  # SVCクラスでrbfを選択
poly_svc = svm.SVC(kernel='poly', gamma='auto', degree=3, C=C).fit(X, y)  # SVCクラスでpolyを選択
lin_svc = svm.LinearSVC(C=C).fit(X, y)  # LinearSVCクラス

x_min, x_max = X[:, 0].min() - 1, X[:, 0].max() + 1
y_min, y_max = X[:, 1].min() - 1, X[:, 1].max() + 1
xx, yy = np.meshgrid(np.arange(x_min, x_max, h),
                     np.arange(y_min, y_max, h))

titles = ['SVCクラスでlinearカーネル選択',
          'LinearSVCクラス (linearカーネル)',
          'SVCクラスでRBFカーネル選択',
          'SVCクラスで3次多項式カーネル選択']

for i, clf in enumerate((svc, lin_svc, rbf_svc, poly_svc)):
    plt.subplot(2, 2, i + 1)  # 4面作る
    plt.subplots_adjust(wspace=0.4, hspace=0.4)

    Z = clf.predict(np.c_[xx.ravel(), yy.ravel()])
    Z = Z.reshape(xx.shape)
    # 区分ごとの色分けを等高線で描画
    plt.contourf(xx, yy, Z, cmap=plt.cm.coolwarm, alpha=0.8)
    plt.scatter(X[:, 0], X[:, 1], c=y, cmap=plt.cm.coolwarm, \
        marker='.')  # 教師データを重ねてプロット
    plt.xlabel('花弁の長さ')
    plt.ylabel('花弁の幅')
    plt.xlim(xx.min(), xx.max())
    plt.ylim(yy.min(), yy.max())
    plt.title(titles[i])
plt.show()
